@@ -14,6 +14,8 @@ EID_EAT_TEST_STOP = 2104301311
 EID_EAT_EA_DEAD = 2104301312
 EID_EAT_LIMIT_ORDERS = 2104301313
 EID_EAT_INVALID_SYMBOL = 2104301314
+EID_EAT_INVALID_DATA_FORMAT = 21043015
+EID_EAT_INVALID_DATA_LENGTH = 21043016
 
 
 class PXErrorCode(Exception):
