@@ -27,6 +27,18 @@ class IndicatiorID():
     TEMA = 17000
     WPR = 18000
 
+class OrderScope():
+    EA = 0
+    EA_VERSION = 200
+    ACCOUNT = 300
+
+class DataScope():
+    EA = 0
+    EA_VERSION = 200
+    ACCOUNT = 300
+    EA_SETTIGNS = 400
+
+
 ErrorID = NewType('ErrorID', int)
 Result = NewType('Result', dict)
 UID = NewType('UID', str)
@@ -41,6 +53,7 @@ class APIStub(abc.ABC):
         OrderType=OrderType,
         OrderCommand=OrderCommand,
         Order=Order,
+        DataScope=DataScope,
     )
 
     def __init__(self):
@@ -55,7 +68,8 @@ class APIStub(abc.ABC):
 
     @abc.abstractmethod
     def Buy(self, volume: float, type=OrderType.MARKET, price=None, stop_loss=None, take_profit=None,
-            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
+            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None,
+            tags=None) -> (ErrorID, OrderResult):
         '''
         Open a buy order.
 
@@ -78,7 +92,8 @@ class APIStub(abc.ABC):
 
     @abc.abstractmethod
     def Sell(self, volume: float, type=OrderType.MARKET, price=None, stop_loss=None, take_profit=None,
-             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
+             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None,
+             tags=None) -> (ErrorID, OrderResult):
         '''
         Open a sell order.
 
@@ -101,7 +116,7 @@ class APIStub(abc.ABC):
 
     @abc.abstractmethod
     def ModifyOrder(self, uid, price=None, stop_loss=None, take_profit=None,
-                     arrow_color=None, expiration=None) -> (ErrorID, OrderResult):
+                     arrow_color=None, expiration=None, tags=None) -> (ErrorID, OrderResult):
         '''
         Modify a order.
 
@@ -120,7 +135,7 @@ class APIStub(abc.ABC):
 
     @abc.abstractmethod
     # def CloseOrder(self, uid, price, volume: float, slippage=None, arrow_color=None) -> (ErrorID, OrderResult):
-    def CloseOrder(self, uid, volume=None, price=None, slippage=None, arrow_color=None) -> (ErrorID, OrderResult):
+    def CloseOrder(self, uid, volume=None, price=None, slippage=None, arrow_color=None, tags=None) -> (ErrorID, OrderResult):
         '''
         Close a order.
 
@@ -264,13 +279,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Close(self, shift=0) -> float:
+    def Close(self, shift=0, symbol=None) -> float:
         '''
         Returns Close price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Close price.
@@ -278,13 +295,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
     #
     @abc.abstractmethod
-    def Open(self, shift=0) -> float:
+    def Open(self, shift=0, symbol=None) -> float:
         '''
         Returns Open price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Open price.
@@ -292,13 +311,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def High(self, shift=0) -> float:
+    def High(self, shift=0, symbol=None) -> float:
         '''
         Returns High price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         High price.
@@ -306,13 +327,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Low(self, shift=0) -> float:
+    def Low(self, shift=0, symbol=None) -> float:
         '''
         Returns Low price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Low price.
@@ -320,13 +343,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Ask(self, shift=0) -> float:
+    def Ask(self, shift=0, symbol=None) -> float:
         '''
         Returns Ask price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Ask price.
@@ -334,13 +359,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Bid(self, shift=0) -> float:
+    def Bid(self, shift=0, symbol=None) -> float:
         '''
         Returns Bid price value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Bid price.
@@ -348,13 +375,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Time(self, shift=0) -> datetime:
+    def Time(self, shift=0, symbol=None) -> datetime:
         '''
         Returns time value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Time.
@@ -362,13 +391,15 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def Volume(self, shift=0) -> float:
+    def Volume(self, shift=0, symbol=None) -> float:
         '''
         Returns volume value for the default symbol with default timeframe and shift.
 
                 Parameters:
                         shift (int): Index of the value taken from the buffer
                         (shift relative to the current the given amount of periods ago).
+                        symbol: The symbol name.
+                                If None returns current symbol.
 
                 Returns:
                         Volume price.
@@ -386,7 +417,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetOpenedOrderUIDs(self, symbol: str = None):
+    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
         '''
         Returns the UIDs of current opened orders.
 
@@ -394,6 +425,10 @@ class APIStub(abc.ABC):
                         symbol: The symbol name.
                                 If None returns current symbol.
                                 If '*' returns all symbols.
+                        scope:
+                            EA: The current ea (default).
+                            ACCOUNT: The current account.
+                            EA_VERSION: The current ea version.
 
                 Returns:
                         The uid list.
@@ -401,7 +436,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetPendingOrderUIDs(self, symbol: str = None):
+    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
         '''
         Returns the UIDs of current pending orders.
 
@@ -409,6 +444,10 @@ class APIStub(abc.ABC):
                         symbol: The symbol name.
                                 If None returns current symbol.
                                 If '*' returns all symbols.
+                        scope:
+                            EA: The current ea (default).
+                            ACCOUNT: The current account.
+                            EA_VERSION: The current ea version.
 
                 Returns:
                         The uid list.
@@ -416,7 +455,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetClosedOrderUIDs(self, symbol: str = None):
+    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
         '''
         Returns the UIDs of current closed orders.
 
@@ -424,6 +463,10 @@ class APIStub(abc.ABC):
                         symbol: The symbol name.
                                 If None returns current symbol.
                                 If '*' returns all symbols.
+                        scope:
+                            EA: The current ea (default).
+                            ACCOUNT: The current account.
+                            EA_VERSION: The current ea version.
 
                 Returns:
                         The uid list.
@@ -746,4 +789,78 @@ class APIStub(abc.ABC):
         '''
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def AcquireLock(self, name, timeout=60) -> bool:
+        '''
+        Acquire a lock
+
+                Parameters:
+                        name : The lock name
+                        timeout : Lock timeout （seconds）
+                Returns:
+                        If True success.
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def ReleaseLock(self, name):
+        '''
+        Release a lock
+
+                Parameters:
+                        name : The lock name
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def Plot(self, series):
+        '''
+        Plot
+                Parameters:
+                        series
+        '''
+        raise NotImplementedError
+
+
+    @abc.abstractmethod
+    def DeleteData(self, name, scope: int = DataScope.EA) -> ErrorID:
+        '''
+        Delete data
+
+                Parameters:
+                        name : The data name
+                        scope : The data scope (current EA settings, EA version, EA, Account)
+                Returns:
+                        The errorid.
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def LoadData(self, name, scope: int = DataScope.EA, format='json'):
+        '''
+        Load data
+
+                Parameters:
+                        name : The data name
+                        scope : The data scope (current EA settings, EA version, EA, Account)
+                        format: Only support JSON.
+                Returns:
+                        data.
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def SaveData(self, name, data, scope: int = DataScope.EA, format='json') -> ErrorID:
+        '''
+        Save data
+
+                Parameters:
+                        name : The data name
+                        scope : The data scope (current EA settings, EA version, EA, Account)
+                        format: Only support JSON.
+                Returns:
+                        The errorid.
+        '''
+        raise NotImplementedError
+
 
