@@ -13,6 +13,7 @@ import pandas as pd
 from pixiu.api.errors import *
 from pixiu.api import (TimeFrame, OrderCommand, order_is_long, order_is_short, order_is_market, order_is_stop,
                        order_is_limit, order_is_pending)
+from pixiu.api.v1 import (OrderScope, DataScope)
 import traceback
 import logging
 log = logging.getLogger(__name__)
@@ -38,7 +39,7 @@ class EATesterPrintCollector(object):
             kwargs['file'] = self
         else:
             self._getattr_(kwargs['file'], 'write')
-        if not self.write_log(*objects):
+        if not self.write_log(*objects, type='ea'):
             print(*objects, **kwargs)
 
 
@@ -50,6 +51,7 @@ class TickMode():
     ONLY_OPEN = 200
 
 
+MAX_DATA_LENGTH = 1048576 #1MB
 class EATester(EABase):
     """EA Tester"""
     def __init__(self, params):
@@ -70,6 +72,7 @@ class EATester(EABase):
         self.script_path = params.get("script_path", None)
         self.log_file = None
         self.log_path = params.get("log_path", None)
+        self.print_log_type = params.get("print_log_type", ['account', 'ea', 'order', 'report'])
         #
         if self.log_path:
             self.log_file = open(self.log_path, mode='at')
@@ -88,13 +91,15 @@ class EATester(EABase):
         self.byte_code = None
         self.orders = None
         self.order_logs = []
+        self.charts_data = []
         self.account_logs = []
         self.balance_dead_line = 0.0
         self.account = params.get("account", None)
         if self.account is None:
             balance = round(float(params['balance']), self.default_digits)
+            equity = balance
             self.account = {'balance': balance,
-                            'equity': balance,
+                            'equity': equity,
                             'margin': 0,
                             'free_margin': balance,
                             'credit': 0.0,
@@ -119,30 +124,70 @@ class EATester(EABase):
                             'margin_so_so': 0.0,
                             'commission': 0.0,
                             }
+        else:
+            default_value = {'equity': self.account['balance'], 'margin': 0, 'free_margin': self.account['balance']}
+            for k in default_value:
+                if k not in self.account:
+                    self.account[k] = default_value[k]
+        #
+        self.print_collection = None
+        #
+        self.current_api = TesterAPI_V1(tester=self, data_source={}, default_symbol=params["symbol"])
+        self.data = {DataScope.EA_VERSION: {}, DataScope.EA: {}, DataScope.ACCOUNT: {}, DataScope.EA_SETTIGNS: {}}
 
+    def delete_data(self, name, scope):
+        self.data[scope].pop(name)
+        return 0
 
+    def load_data(self, name, scope, format='json'):
+        if format != 'json':
+            return None
+        data = self.data[scope].get(name, None)
+        if data is None:
+            return None
+        return json.loads(data)
+
+    def save_data(self, name, data, scope, format='json'):
+        if format != 'json':
+            return EID_EAT_INVALID_DATA_FORMAT
+        if data is not None:
+            data = json.dumps(data)
+            if len(data) > MAX_DATA_LENGTH:
+                return EID_EAT_INVALID_DATA_LENGTH
+        self.data[scope][name] = data
+        return EID_OK
+
+    def init_report_data(self):
         self.report = {
                         'init_balance': {'value': self.account['balance'], 'desc': 'Init Balance'}, #
-                        'ticks': {'value': 0, 'desc': 'Ticks'}, #
-                        'total_net_profit': {'value': 0, 'desc': 'Total net profit'}, #
+                        'symbol': {'value': self.symbol, 'desc': 'Symbol', 'type': 'str'}, #
+                        'currency': {'value': self.account['currency'], 'desc': 'Currency', 'type': 'str'}, #
+                        'leverage': {'value': self.account['leverage'], 'desc': 'Leverage'}, #
+                        'spread_point': {'value': self.spread_point, 'desc': 'Spread Points'}, #
+                        'ticks': {'value': 0, 'desc': 'Ticks', 'precision': 0}, #
+                        'balance': {'value': 0, 'desc': 'Balance'}, #
+                        'total_net_profit': {'value': 0, 'desc': 'Total Net Profit'}, #
+                        'total_net_profit_rate': {'value': 0, 'desc': 'Total Net Profit Rate', 'type': '%'}, #
                         'absolute_drawdown': {'value': 0, 'desc': 'Absolute Drawdown'}, #
                         'max_drawdown': {'value': 0, 'desc': 'Max Drawdown'}, #
-                        'total_trades': {'value': 0, 'desc': 'Total Trades'},#
-                        'profit_trades': {'value': 0, 'desc': 'Profit Trades'},#
+                        'max_drawdown_rate': {'value': 0, 'desc': 'Max Drawdown Rate', 'type': '%'}, #
+                        'total_trades': {'value': 0, 'desc': 'Total Trades', 'precision': 0},#
+                        'profit_trades': {'value': 0, 'desc': 'Profit Trades', 'precision': 0},#
+                        'win_rate': {'value': 0, 'desc': 'Win Rate', 'type': '%'},  #
                         'trade_max_profit': {'value': 0, 'desc': 'Trade Max Profit'}, #
                         'trade_avg_profit': {'value': 0, 'desc': 'Trade Avg Profit'}, #
-                        'trade_max_loss': {'value': 0, 'desc': 'Trade Max Loss'}, #
+                        'trade_max_loss': {'value': np.nan, 'desc': 'Trade Max Loss'}, #
                         'trade_avg_loss': {'value': 0, 'desc': 'Trade Avg Loss'}, #
-                        'loss_trades': {'value': 0, 'desc': 'Loss Trades'}, #
+                        'loss_trades': {'value': 0, 'desc': 'Loss Trades', 'precision': 0}, #
                         'gross_profit': {'value': 0, 'desc': 'Gross Profit'}, #
                         'gross_loss': {'value': 0, 'desc': 'Gross Loss'}, #
-                        'short_positions': {'value': 0, 'desc': 'Short Positions'}, #
-                        'short_positions_win': {'value': 0, 'desc': 'Short Positions Win'}, #
-                        'long_positions': {'value': 0, 'desc': 'Long Positions'}, #
-                        'long_positions_win': {'value': 0, 'desc': 'Long Positions Win'}, #
-                        'max_consecutive_wins': {'value': 0, 'desc': 'Max Consecutive Wins'}, #
+                        'short_positions': {'value': 0, 'desc': 'Short Positions', 'precision': 0}, #
+                        'short_positions_win': {'value': 0, 'desc': 'Short Positions Win', 'precision': 0}, #
+                        'long_positions': {'value': 0, 'desc': 'Long Positions', 'precision': 0}, #
+                        'long_positions_win': {'value': 0, 'desc': 'Long Positions Win', 'precision': 0}, #
+                        'max_consecutive_wins': {'value': 0, 'desc': 'Max Consecutive Wins', 'precision': 0}, #
                         'max_consecutive_wins_money': {'value': 0, 'desc': 'Max Consecutive Wins Money'}, #
-                        'max_consecutive_losses': {'value': 0, 'desc': 'Max Consecutive Losses'}, #
+                        'max_consecutive_losses': {'value': 0, 'desc': 'Max Consecutive Losses', 'precision': 0}, #
                         'max_consecutive_losses_money': {'value': 0, 'desc': 'Max Consecutive Losses Money'}, #
         }
         self.temp = {
@@ -150,11 +195,12 @@ class EATester(EABase):
             'consecutive_wins_money': 0,
             'consecutive_losses': 0,
             'consecutive_losses_money': 0,
+            'max_drawdown': 0,
+            'max_drawdown_rate': 0,
+            'account_max_equity': self.account['equity'],
+            'account_min_equity': self.account['equity'],
+            'account_min_balance': self.account['balance'],
         }
-        self.print_collection = None
-        #
-        self.current_api = TesterAPI_V1(tester=self, data_source={}, default_symbol=params["symbol"])
-
 
     def get_print_factory(self, _getattr_=None):
         """print factory"""
@@ -162,15 +208,33 @@ class EATester(EABase):
             self.print_collection = EATesterPrintCollector(_getattr_, self.write_log)
         return self.print_collection
 
-    def write_log(self, log_str, print_=True):
-        if print_:
-            print(log_str)
+    def write_log(self, *args, **kwargs):
+        log_type = kwargs.get('type', 'eat')
+        if log_type == 'eat' or log_type in self.print_log_type:
+            print(*args)
         #
         if self.log_file:
+            log_str = ''
+            for a in args:
+                log_str = f'{log_str} {a}'
             self.log_file.write(str(log_str))
+            # self.log_file.write(*args)
             self.log_file.write('\n')
         return True
-
+    #
+    # def write_log(self, *args, **kwargs):
+    #     print_ = kwargs.get('print_', True)
+    #     if print_:
+    #         print(*args)
+    #     #
+    #     if self.log_file:
+    #         log_str = ''
+    #         for a in args:
+    #             log_str = f'{log_str} {a}'
+    #         self.log_file.write(str(log_str))
+    #         # self.log_file.write(*args)
+    #         self.log_file.write('\n')
+    #     return True
 
     def get_account_data(self, timeframe):
         """"""
@@ -250,6 +314,10 @@ class EATester(EABase):
         log_dict['id'] = len(self.order_logs) + 1
         self.order_logs.append(log_dict)
 
+    def plot(self, chart_name, series):
+        self.charts_data.append(dict(cn=chart_name, data=series))
+        # self.charts_data.append(dict(cn="default", data=series))
+
     def add_account_log(self, log_dict):
         """Add account log"""
         self.account_logs.append(log_dict)
@@ -340,7 +408,7 @@ class EATester(EABase):
         if margin > self.account['balance'] - self.account['margin']:
             return EID_EAT_MARGIN_CALL, -1
         new_order['uid'] = str(new_order['ticket'])
-        new_order['comment'] = f"cuid#{new_order['uid']}|"
+        new_order['comment'] = f"uid#{new_order['uid']}|"
         new_order['margin'] = margin
         commission = 0.0
         #commissions
@@ -489,8 +557,7 @@ class EATester(EABase):
 
         return ret
 
-    def get_order_dict(self, symbol, status="opened", script_name=None):
-        """"""
+    def get_order_dict(self, symbol, status="opened", scope=OrderScope.EA):
         symbol = self.symbol if symbol is None else symbol
         order_list = self.orders.get(status, {})
         if symbol == '*':
@@ -610,7 +677,7 @@ class EATester(EABase):
         return str(self.orders['counter'] + 1)
 
     def open_order(self, symbol, cmd, price, volume, stop_loss, take_profit, comment=None, ext_check_open_range=0,
-                       ext_check_order_hold_count=0, magic_number=None, slippage=None, arrow_color=None):
+                       ext_check_order_hold_count=0, magic_number=None, slippage=None, arrow_color=None, tags=None):
         """"""
         order_uid = None
         account = self.account
@@ -624,7 +691,7 @@ class EATester(EABase):
         order_dict = dict(ticket=self.__new_ticket__(), symbol=symbol, cmd=cmd, open_price=price,
                          volume=volume, stop_loss=stop_loss, take_profit=take_profit, margin=0, comment=comment,
                          magic_number=magic_number, open_time=self.current_time(), commission=0,
-                         close_time=None, close_price=np.nan, profit=0.0)
+                         close_time=None, close_price=np.nan, profit=0.0, tags=tags)
         errid = self.__valid_order__(0, order_dict, self.Close())
         if errid != EID_OK:
             return errid, dict(order_uid=order_uid, command_uid=None, sync=True)
@@ -649,17 +716,21 @@ class EATester(EABase):
     #
 
     def modify_order(self, order_uid, price, stop_loss, take_profit, comment=None, arrow_color=None,
-                     expiration=None):
-        """"""
+                     expiration=None, tags=None):
+        """modify_order"""
         order_dict = self.get_order(order_uid=order_uid)
         if order_dict is None:
             return EID_EAT_INVALID_ORDER_TICKET, dict(order_uid=order_uid, command_uid=None, sync=True)
         if order_dict['symbol'] != self.symbol:
             return EID_EAT_INVALID_SYMBOL, dict(order_uid=order_uid, command_uid=None, sync=True)
         order_tmp = order_dict.copy()
-        order_tmp['stop_loss'] = stop_loss
-        order_tmp['take_profit'] = take_profit
+        if stop_loss is not None:
+            order_tmp['stop_loss'] = stop_loss
+        if take_profit is not None:
+            order_tmp['take_profit'] = take_profit
         order_tmp['comment'] = comment
+        if tags is not None:
+            order_tmp['tags'] = tags
         #
         close_price = self.Close()
         if order_is_market(order_dict['cmd']):
@@ -678,22 +749,76 @@ class EATester(EABase):
             return errid, dict(order_uid=order_uid, command_uid=None, sync=True)
         #
         order_dict['open_price'] = order_tmp['open_price']
-        order_dict['stop_loss'] = order_tmp['stop_loss']
-        order_dict['take_profit'] = order_tmp['take_profit']
+        if 'stop_loss' in order_tmp:
+            order_dict['stop_loss'] = order_tmp['stop_loss']
+        if 'take_profit' in order_tmp:
+            order_dict['take_profit'] = order_tmp['take_profit']
         order_dict['comment'] = order_tmp['comment']
         self.__modify_order__(order_dict)
 
         #
-        self.add_order_log(dict(ticket=order_dict['ticket'],
+        order_log = dict(uid=order_uid, ticket=order_dict['ticket'],
                                  time=str(datetime.fromtimestamp(self.current_time())),
                                  type="MODIFY",
-                                 size=order_dict['volume'], price=round(price, self.price_digits),
-                                 stop_loss=round(stop_loss, self.price_digits),
-                                 take_profit=round(take_profit, self.price_digits),
+                                 volume=order_dict['volume'], price=round(price, self.price_digits),
+                                 # stop_loss=round(stop_loss, self.price_digits),
+                                 # take_profit=round(take_profit, self.price_digits),
                                  profit=round(order_dict['profit'], self.default_digits),
                                  balance=round(self.account["balance"] + order_dict['profit'], self.default_digits),
-                                 comment=comment))
+                                 comment=comment)
+        if stop_loss is not None:
+            order_log['stop_loss'] = round(stop_loss, self.price_digits)
+        if take_profit is not None:
+            order_log['take_profit'] = round(take_profit, self.price_digits),
+        self.add_order_log(order_log)
         return EID_OK, dict(order_uid=order_uid, command_uid=None, sync=True)
+    #
+    # def modify_order(self, order_uid, price, stop_loss, take_profit, comment=None, arrow_color=None,
+    #                  expiration=None):
+    #     """"""
+    #     order_dict = self.get_order(order_uid=order_uid)
+    #     if order_dict is None:
+    #         return EID_EAT_INVALID_ORDER_TICKET, dict(order_uid=order_uid, command_uid=None, sync=True)
+    #     if order_dict['symbol'] != self.symbol:
+    #         return EID_EAT_INVALID_SYMBOL, dict(order_uid=order_uid, command_uid=None, sync=True)
+    #     order_tmp = order_dict.copy()
+    #     order_tmp['stop_loss'] = stop_loss
+    #     order_tmp['take_profit'] = take_profit
+    #     order_tmp['comment'] = comment
+    #     #
+    #     close_price = self.Close()
+    #     if order_is_market(order_dict['cmd']):
+    #         if price is None:
+    #             price = close_price
+    #         #
+    #     else:
+    #         if price is None:
+    #             price = close_price
+    #         else:
+    #             order_tmp['price'] = price
+    #             order_tmp['open_price'] = price
+    #     #
+    #     errid = self.__valid_order__(1, order_tmp, close_price)
+    #     if errid != EID_OK:
+    #         return errid, dict(order_uid=order_uid, command_uid=None, sync=True)
+    #     #
+    #     order_dict['open_price'] = order_tmp['open_price']
+    #     order_dict['stop_loss'] = order_tmp['stop_loss']
+    #     order_dict['take_profit'] = order_tmp['take_profit']
+    #     order_dict['comment'] = order_tmp['comment']
+    #     self.__modify_order__(order_dict)
+    #
+    #     #
+    #     self.add_order_log(dict(uid=order_uid, ticket=order_dict['ticket'],
+    #                              time=str(datetime.fromtimestamp(self.current_time())),
+    #                              type="MODIFY",
+    #                              volume=order_dict['volume'], price=round(price, self.price_digits),
+    #                              stop_loss=round(stop_loss, self.price_digits),
+    #                              take_profit=round(take_profit, self.price_digits),
+    #                              profit=round(order_dict['profit'], self.default_digits),
+    #                              balance=round(self.account["balance"] + order_dict['profit'], self.default_digits),
+    #                              comment=comment))
+    #     return EID_OK, dict(order_uid=order_uid, command_uid=None, sync=True)
 
     def __order_close_price__(self, order_dict):
         if order_is_market(order_dict['cmd']):
@@ -701,9 +826,8 @@ class EATester(EABase):
         else:
             return 0
 
-
     def close_order(self, order_uid, volume, price, slippage=None, comment=None, arrow_color=None,
-                    update_report_func=None):
+                    update_report_func=None, tags=None):
         """Close order"""
         order_dict = self.get_order(order_uid=order_uid)
         if order_dict is None:
@@ -721,9 +845,11 @@ class EATester(EABase):
             if volume > order_dict['volume']:
                 return EID_EAT_INVALID_ORDER_VOLUME, dict(order_uid=order_uid, command_uid=None, sync=True)
             #
-            errid  = self.__valid_order__(2, order_dict, price)
+            errid = self.__valid_order__(2, order_dict, price)
             if errid != EID_OK:
                 return errid, dict(order_uid=order_uid, command_uid=None, sync=True)
+            if tags is not None:
+                order_dict['tags'] = tags
             order_dict['comment'] = comment
             order_dict['close_time'] = close_time
             order_dict['close_price'] = price
@@ -770,37 +896,72 @@ class EATester(EABase):
     def wait_command(self, uid, timeout=120):
         return 0, {}
 
+    def acquire_lock(self, name, timeout=60):
+        return True
+
+    def release_lock(self, name):
+        pass
+
     #
-    def Ask(self, shift=0):
+    def Ask(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         ask = self.tick_info['a'][self.current_tick_index - shift]
         if ask == 0:
             ask = self.Bid(shift) + self.spread_calculated
         return ask
 
-    def Bid(self, shift=0):
+    def Bid(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         bid = self.tick_info['b'][self.current_tick_index - shift]
         if bid == 0:
             bid = self.Close(shift)
         return bid
 
-    def Close(self, shift=0):
+    def Close(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         return self.tick_info['c'][self.current_tick_index - shift]
 
-    def Open(self, shift=0):
+    def Open(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         return self.tick_info['o'][self.current_tick_index - shift]
 
-    def High(self, shift=0):
+    def High(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         return self.tick_info['h'][self.current_tick_index - shift]
 
-    def Low(self, shift=0):
+    def Low(self, shift=0, symbol=None):
+        if symbol is not None and symbol != self.symbol:
+            return None
         return self.tick_info['l'][self.current_tick_index - shift]
 
-    def Time(self, shift=0) -> datetime:
+    def Time(self, shift=0, symbol=None) -> datetime:
+        if symbol is not None and symbol != self.symbol:
+            return None
         return datetime.fromtimestamp(self.tick_info['t'][self.current_tick_index - shift])
 
-    def Volume(self, shift=0) -> float:
+    def Volume(self, shift=0, symbol=None) -> float:
+        if symbol is not None and symbol != self.symbol:
+            return None
         return self.tick_info['v'][self.current_tick_index - shift]
     #
+    def __update_report_max_dd__(self):
+        #calculate maxdd: https://www.investopedia.com/terms/m/maximum-drawdown-mdd.asp
+        if self.temp['account_min_equity'] < self.report['init_balance']['value']:
+            self.temp['max_drawdown'] = self.temp['account_min_equity'] - self.temp['account_max_equity']
+        if self.temp['account_max_equity'] > 0:
+            self.temp['max_drawdown_rate'] = self.temp['max_drawdown'] / self.temp['account_max_equity']
+        else:
+            self.temp['max_drawdown_rate'] = 0
+        #
+        if self.report['max_drawdown']['value'] > self.temp['max_drawdown']:
+            self.report['max_drawdown']['value'] = self.temp['max_drawdown']
+            self.report['max_drawdown_rate']['value'] = self.temp['max_drawdown_rate']
+
     def __update_report__(self, is_long, profit, trades=1):
         if profit == 0:
             return
@@ -822,21 +983,46 @@ class EATester(EABase):
             self.temp['consecutive_wins_money'] = self.temp['consecutive_wins_money'] + profit
             self.temp['consecutive_losses'] = 0
             self.temp['consecutive_losses_money'] = 0
+            #self.account['currency']
         else:
             self.report['loss_trades']['value'] += trades
             self.report['gross_loss']['value'] = self.report['gross_loss']['value'] + profit
-            self.report['max_drawdown']['value'] = self.report['gross_loss']['value']
             self.report['trade_avg_loss']['value'] = self.report['gross_loss']['value'] / self.report['loss_trades'][
                 'value']
 
-            if self.report['trade_max_loss']['value'] < profit:
+            if np.isnan(self.report['trade_max_loss']['value']) or self.report['trade_max_loss']['value'] > profit:
                 self.report['trade_max_loss']['value'] = profit
 
             self.temp['consecutive_losses'] += trades
             self.temp['consecutive_losses_money'] = self.temp['consecutive_losses_money'] + profit
             self.temp['consecutive_wins'] = 0
             self.temp['consecutive_wins_money'] = 0
+            #
         #
+        #max dd
+        if self.temp['account_max_equity'] < self.account['equity']:
+            # #calculate maxdd: https://www.investopedia.com/terms/m/maximum-drawdown-mdd.asp
+            self.__update_report_max_dd__()
+            #
+            self.temp['account_max_equity'] = self.account['equity']
+            self.temp['account_min_equity'] = self.account['equity']
+        else:
+            if self.temp['account_min_equity'] > self.account['equity']:
+                self.__update_report_max_dd__()
+                self.temp['account_min_equity'] = self.account['equity']
+        #
+
+        #
+        if self.temp['account_min_balance'] > self.account['balance']:
+            self.temp['account_min_balance'] = self.account['balance']
+            self.report['absolute_drawdown']['value'] = self.temp['account_min_balance'] - self.report['init_balance']['value']
+
+        #
+        # self.report['max_drawdown']['value'] = self.temp['account_max_equity'] - self.temp['account_min_equity']
+        # if self.temp['account_max_equity'] > 0:
+        #     self.report['max_drawdown_rate']['value'] = self.report['max_drawdown']['value'] / self.temp['account_max_equity']
+        # else:
+        #     self.report['max_drawdown_rate']['value'] = 0
         if self.report['max_consecutive_wins']['value'] < self.temp['consecutive_wins']:
             self.report['max_consecutive_wins']['value'] = self.temp['consecutive_wins']
         if self.report['max_consecutive_wins_money']['value'] < self.temp['consecutive_wins_money']:
@@ -847,8 +1033,81 @@ class EATester(EABase):
             self.report['max_consecutive_losses_money']['value'] = self.temp['consecutive_losses_money']
         self.report['total_net_profit']['value'] = self.report['gross_profit']['value'] - abs(
             self.report['gross_loss']['value'])
-        if self.report['total_net_profit']['value'] < 0:
-            self.report['absolute_drawdown']['value'] = abs(self.report['total_net_profit']['value'])
+        #
+        if self.report['total_trades']['value'] > 0:
+            self.report['win_rate']['value'] = self.report['profit_trades']['value'] / self.report['total_trades']['value']
+        #Total net profit rate
+        if self.report['init_balance']['value'] > 0:
+            self.report['total_net_profit_rate']['value'] = self.report['total_net_profit']['value'] / self.report['init_balance']['value']
+        self.report['balance']['value'] = self.account['balance']
+
+    # def __update_report__(self, is_long, profit, trades=1):
+    #     if profit == 0:
+    #         return
+    #     profit = round(profit, self.default_digits)
+    #     if profit > 0:
+    #         self.report['profit_trades']['value'] += trades
+    #         self.report['gross_profit']['value'] = self.report['gross_profit']['value'] + profit
+    #         self.report['trade_avg_profit']['value'] = self.report['gross_profit']['value'] / \
+    #                                                    self.report['profit_trades']['value']
+    #
+    #         if self.report['trade_max_profit']['value'] < profit:
+    #             self.report['trade_max_profit']['value'] = profit
+    #         if is_long:
+    #             self.report['long_positions_win']['value'] += trades
+    #         else:
+    #             self.report['short_positions_win']['value'] += trades
+    #
+    #         self.temp['consecutive_wins'] += trades
+    #         self.temp['consecutive_wins_money'] = self.temp['consecutive_wins_money'] + profit
+    #         self.temp['consecutive_losses'] = 0
+    #         self.temp['consecutive_losses_money'] = 0
+    #         #self.account['currency']
+    #     else:
+    #         self.report['loss_trades']['value'] += trades
+    #         self.report['gross_loss']['value'] = self.report['gross_loss']['value'] + profit
+    #         self.report['trade_avg_loss']['value'] = self.report['gross_loss']['value'] / self.report['loss_trades'][
+    #             'value']
+    #
+    #         if np.isnan(self.report['trade_max_loss']['value']) or self.report['trade_max_loss']['value'] > profit:
+    #             self.report['trade_max_loss']['value'] = profit
+    #
+    #         self.temp['consecutive_losses'] += trades
+    #         self.temp['consecutive_losses_money'] = self.temp['consecutive_losses_money'] + profit
+    #         self.temp['consecutive_wins'] = 0
+    #         self.temp['consecutive_wins_money'] = 0
+    #         #
+    #     #
+    #     #
+    #     if self.temp['account_max_equity'] < self.account['equity']:
+    #         self.temp['account_max_equity'] = self.account['equity']
+    #     if self.temp['account_min_equity'] > self.account['equity']:
+    #         self.temp['account_min_equity'] = self.account['equity']
+    #     #
+    #     self.report['max_drawdown']['value'] = self.temp['account_max_equity'] - self.temp['account_min_equity']
+    #     if self.temp['account_max_equity'] > 0:
+    #         self.report['max_drawdown_rate']['value'] = self.report['max_drawdown']['value'] / self.temp['account_max_equity']
+    #     else:
+    #         self.report['max_drawdown_rate']['value'] = 0
+    #     if self.report['max_consecutive_wins']['value'] < self.temp['consecutive_wins']:
+    #         self.report['max_consecutive_wins']['value'] = self.temp['consecutive_wins']
+    #     if self.report['max_consecutive_wins_money']['value'] < self.temp['consecutive_wins_money']:
+    #         self.report['max_consecutive_wins_money']['value'] = self.temp['consecutive_wins_money']
+    #     if self.report['max_consecutive_losses']['value'] < self.temp['consecutive_losses']:
+    #         self.report['max_consecutive_losses']['value'] = self.temp['consecutive_losses']
+    #     if self.report['max_consecutive_losses_money']['value'] > self.temp['consecutive_losses_money']:
+    #         self.report['max_consecutive_losses_money']['value'] = self.temp['consecutive_losses_money']
+    #     self.report['total_net_profit']['value'] = self.report['gross_profit']['value'] - abs(
+    #         self.report['gross_loss']['value'])
+    #     if self.report['total_net_profit']['value'] < 0:
+    #         self.report['absolute_drawdown']['value'] = abs(self.report['total_net_profit']['value'])
+    #     #
+    #     if self.report['total_trades']['value'] > 0:
+    #         self.report['win_rate']['value'] = self.report['profit_trades']['value'] / self.report['total_trades']['value']
+    #     #Total net profit rate
+    #     if self.report['init_balance']['value'] > 0:
+    #         self.report['total_net_profit_rate']['value'] = self.report['total_net_profit']['value'] / self.report['init_balance']['value']
+    #
 
     def __calculate_pip_with_quote_currency__(self, price):
         return 1.0
@@ -890,9 +1149,11 @@ class EATester(EABase):
         self.symbol_data = {}
         self.current_tick_index = 0
         self.update_log_time = 0
+        self.update_charts_time = 0
         self.last_update_order_log_index = 0
         self.last_update_account_log_index = 0
         self.last_update_print_log_index = 0
+        self.last_update_charts_data_index = 0
         self.cache = {}
         #
         self.order_logs = []
@@ -919,7 +1180,7 @@ class EATester(EABase):
         sp = self.get_symbol_properties(self.symbol)
         if self.spread_point is None:
             self.spread_point = int(sp['spread'])
-        self.price_digits =  int(sp['digits'])
+        self.price_digits = int(sp['digits'])
         self.spread_calculated = abs(self.spread_point * sp['point'])
         #
         if self.account['currency'] is None:
@@ -930,6 +1191,8 @@ class EATester(EABase):
         else:
             self.__calculate_pip__ = self.__calculate_pip_with_quote_currency__
         self.stop = False
+        #
+        self.init_report_data()
 
     def get_symbol_properties(self, symbol):
         ''''''
@@ -967,7 +1230,6 @@ class EATester(EABase):
             for r in result:
                 self.__active_pending_order__(str(r['oid']), price, comment='open')
 
-
         #update opened orders
         order_report = {True: dict(profit=0.0, trades=0), False: dict(profit=0.0, trades=0)}
 
@@ -980,9 +1242,12 @@ class EATester(EABase):
             # result = ds[(ds['sl_p'] <= 0) | (ds['tp_p'] >= 0)]
             result = ds[((ds['sl'] > 0) & (ds['sl_p'] <= 0)) | ((ds['tp'] > 0) & (ds['tp_p'] >= 0))]
             for r in result:
-                self.close_order(str(r['oid']), price, r['v'],
+                self.close_order(str(r['oid']), r['v'], None,
                                  comment='sl' if r['sl_p'] <= 0 else 'tp',
                                  update_report_func=update_report_func)
+                # self.close_order(str(r['oid']), r['v'], price,
+                #                  comment='sl' if r['sl_p'] <= 0 else 'tp',
+                #                  update_report_func=update_report_func)
 
         #update reports
         for key in order_report:
@@ -995,7 +1260,7 @@ class EATester(EABase):
         symbol_orders = self.get_order_dict(self.symbol).copy()
         for order_uid in symbol_orders:
             order_dict = self.get_order(order_uid=order_uid)
-            self.close_order(order_uid, price, order_dict['volume'], comment=comment)
+            self.close_order(order_uid, order_dict['volume'], price, comment=comment)
     #
     def __update_account_log(self, ticket):
         ''''''
@@ -1081,8 +1346,9 @@ class EATester(EABase):
         errid = EID_OK
         update_log_task = None
         try:
-            self.write_log(f"\n\n == Execute PiXiu backtesting: {datetime.now()}, ticket: {ticket}, symbol: {self.symbol}, period: {self.start_time} - {self.end_time}, "
-                           f"tick_timeframe: {self.tick_timeframe}, tick_mode: {self.tick_mode} == \n\n")
+            test_start_time = datetime.now()
+            self.write_log(f"\n\n == PiXiu Backtesting Start: {test_start_time}, Ticket: {ticket}, Symbol: {self.symbol}, Period: {self.start_time} - {self.end_time}, "
+                           f"Timeframe: {self.tick_timeframe}, Mode: {self.tick_mode} == \n\n")
             self.init_data()
             self.update_log_task_running = True
             self.on_pre_load_ticks()
@@ -1092,7 +1358,7 @@ class EATester(EABase):
 
             count = self.tick_info.size
 
-            self.write_log(f"execute start ... count={count}, tick_max_count={self.tick_max_count}")
+            self.write_log(f"Tick Count: {count}, Tick Max Count: {self.tick_max_count}")
             status = dict(current=self.current_tick_index, max=count, errid=0)
             self.report['ticks']['value'] = status['max']
             self.on_execute_status(ticket, status)
@@ -1141,7 +1407,9 @@ class EATester(EABase):
                 traceback.print_exc()
             self.on_end_execute()
 
-            self.write_log(f"\n\n == Execute PiXiu backtesting end: {datetime.now()}, ticket: {ticket} == \n\n")
+            # self.write_log(f"\n\n == Execute PiXiu backtesting end: {datetime.now()}, ticket: {ticket} == \n\n")
+            test_end_time = datetime.now()
+            self.write_log(f"\n\n == PiXiu Backtesting End: {test_end_time}, Total Time: {(test_end_time-test_start_time).total_seconds()} sec, Ticket: {ticket} == \n\n")
             status = {}
             if exception_msg is not None:
                 status["exception"] = exception_msg
